#include <stdio.h>
#include <imsls.h>
/* ********************************************************************
* Three Layer Feed-Forward Network with 4 inputs, all 
* continuous, and 3 classification categories.
*     
* This is perhaps the best known database to be found in the pattern
*     recognition literature.  Fisher's paper is a classic in the 
*     field. The data set contains 3 classes of 50 instances each,
*     where each class refers to a type of iris plant.  One class is
*     linearly separable from the other 2; the latter are NOT linearly
*     separable from each other.
*
*  Predicted attribute: class of iris plant.
*     1=Iris Setosa, 2=Iris Versicolour, and 3=Iris Virginica
*
*  Input Attributes (4 Continuous Attributes)
*     X1: Sepal length, 
*     X2: Sepal width, 
*     X3: Petal length, 
* and X4: Petal width
**********************************************************************/
int main(){  
    float *classProb;
    int i, j;
    int n_patterns    =150;
    int n_inputs      =4;  /* four inputs, all continuous */
    int n_nominal     =0;  /* no nominal input attributes */
    int n_continuous  =4;  /* one continuous input attribute */
    int n_outputs     =3;  /* total number of output perceptrons */
    int *predicted_class;
    int act_fcn[3] = {1, 1, 1};
    int classification[150];
    float unscaledX[150], scaledX[150];
    float contAtt[4*150];
    float *irisData;
    float mean[4], s[4], center, spread;
    char *colLabels[] = {"Pattern", "Class=0", "Class=1", "Class=2"};
    char filename[] = "iris_classification.txt";
    char prtLabel[] ="\nPredicted_Class  |   P(0)     P(1)     P(2)";
    char dashes[] = "-----------------------------------------------";
    /* Declare mlff network structure */
    Imsls_f_NN_Network *network;

    printf("******************************************************\n");
    printf(" IRIS CLASSIFICATION EXAMPLE - PATTERN CLASSIFICATION \n"); 
    printf("******************************************************\n");
    irisData = imsls_f_data_sets(3,0);

    /*
    * Setup the continuous attribute input array, contAtt[], and the 
    * network target classification array, classification[], using 
    * the above raw data matrix.
    */
    for(i=0; i < n_patterns; i++){
        classification[i] = (int)irisData[i*5]-1;
        for(j=1; j<5; j++){
            contAtt[i*4+j-1] = irisData[i*5+j];
        }
    }
    /* Scale continuous input attributes using z-score method */
    for(j=0; j<n_continuous; j++){
        for(i=0; i<n_patterns; i++) 
            unscaledX[i] = contAtt[i*n_continuous+j];
        imsls_f_scale_filter(n_patterns, unscaledX, 2, 
            IMSLS_RETURN_USER, scaledX,
            IMSLS_RETURN_CENTER_SPREAD, &center, &spread, 0);
        for(i=0; i<n_patterns; i++) 
            contAtt[i*n_continuous+j] = scaledX[i];
        mean[j] = center;
        s[j]    = spread;
    }
    printf("Scale Parameters: \n");
    for(j=0; j<n_continuous; j++){
        printf("Var %d Mean = %f S = %f \n",j+1, mean[j], s[j]);
    } 

    network = imsls_f_mlff_network_read(filename, IMSLS_PRINT,0);

    /* Use pattern classification routine to classify training 
     * patterns using trained network. 
     */
    classProb = imsls_f_mlff_pattern_classification (network,
        n_patterns, n_nominal, n_continuous, NULL, contAtt, 
        IMSLS_PREDICTED_CLASS, &predicted_class, 0);

    /* Print class predictions */
    printf("\n%s\n",prtLabel);
    printf("%s\n",dashes);
    for(i=0; i<n_patterns; i++){
        printf("       %d   ", predicted_class[i]);
        printf("      | %f %f %f \n", classProb[i*3], classProb[i*3+1], 
            classProb[i*3+2]);
        if(i==49 || i==99){
            printf("\n%s\n",prtLabel);
            printf("%s\n",dashes);
        }
    }
}
